/**
 * EasyBeans
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: WarAnnotationDeploymentAnalyzer.java 2750 2008-04-02 09:15:37Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.deployment.annotations.analyzer;

import java.util.List;

import org.ow2.easybeans.deployment.annotations.analyzer.configurator.metadata.war.WarArchiveMetadataConfigurator;
import org.ow2.easybeans.deployment.metadata.war.WarArchiveMetadata;
import org.ow2.util.deployment.annotations.analyzer.AnalyzerException;
import org.ow2.util.deployment.annotations.analyzer.ArchiveScanner;
import org.ow2.util.deployment.annotations.analyzer.ArchiveScannerFactory;
import org.ow2.util.ee.deploy.api.archive.IArchive;

/**
 * This class fill metadata for Servlet classes.
 * @author Florent Benoit
 */
public class WarAnnotationDeploymentAnalyzer {

    /**
     * Metadata representing the parsed war file.
     */
    private WarArchiveMetadataConfigurator warArchiveMetadataConfigurator = null;

    /**
     * The scanner.
     */
    private ArchiveScannerFactory archiveScannerFactory;

    /**
     * The archive to scan.
     */
    private IArchive archive;

    /**
     * ClassLoader used to read classes.
     */
    private ClassLoader classLoader;

    /**
     * Constructor.<br>
     * Archive which will be used when analyzing.
     * @param archive the archive to analyze.
     * @param classLoader the classloader used to read the classes.
     */
    public WarAnnotationDeploymentAnalyzer(final IArchive archive, final ClassLoader classLoader) {
        this(null, archive, classLoader);
    }

    /**
     * Constructor.<br>
     * Archive which will be used when analyzing.
     * @param archiveScannerFactory factory to create scanner.
     * @param archive the archive to analyze.
     * @param classLoader the classloader used to read the classes.
     */
    public WarAnnotationDeploymentAnalyzer(final ArchiveScannerFactory archiveScannerFactory, final IArchive archive,
            final ClassLoader classLoader) {
        this.archiveScannerFactory = archiveScannerFactory;
        this.archive = archive;
        this.classLoader = classLoader;
        this.warArchiveMetadataConfigurator = new WarArchiveMetadataConfigurator();
    }

    /**
     * Analyzes the given set of classes and fill metadata struct.
     * @param classNames the class names to analyze
     * @throws AnalyzerException if analyze of archive fails
     */
    public void analyze(final List<String> classNames) throws AnalyzerException {
        // TODO: Analyze super classes

        // Analyze the set of given classes

        if (this.archiveScannerFactory == null) {
            this.archiveScannerFactory = ArchiveScannerFactory.getReferenceImplementationInstance();
        }
        ArchiveScanner archiveScanner = this.archiveScannerFactory.createArchiveScanner(this.archive, new WarArchiveHelper(
                classNames, this.classLoader));
        archiveScanner.addArchiveConfigurator(this.warArchiveMetadataConfigurator);

        archiveScanner.scanCollection(classNames);
        archiveScanner.endScan();
    }

    /**
     * @return struct (metadata) filled by this analyzer
     */
    public WarArchiveMetadata getWarArchiveMetadata() {
        return this.warArchiveMetadataConfigurator.getWarArchiveMetadata();
    }
}
